@@ -1,7 +1,3 @@
-#
-# $Id: AuthTicket.pm 40 2005-04-18 19:48:25Z mschout $
-#
-
 package Apache2::AuthTicket;
 
 use strict;
@@ -14,11 +10,11 @@ use Apache2::Connection;
 use Apache2::ServerUtil;
 use DBI ();
 use SQL::Abstract;
-use Digest::MD5 qw(md5_hex);
+use Apache::AuthTicket::Util qw(compare_password);
 
 use constant DEBUGGING => 0;
 
-our $VERSION = '0.40';
+our $VERSION = '0.90';
 
 # configuration items
 # PerlSetVar FooTicketDB  dbi:Pg:dbname=template1
@@ -40,7 +36,9 @@ our $VERSION = '0.40';
     TicketUserTable       => 'users:usrname:passwd',
     TicketPasswordStyle   => 'cleartext',
     TicketSecretTable     => 'ticketsecrets:sec_data:sec_version',
-    TicketLoginHandler    => '/login'
+    TicketLoginHandler    => '/login',
+    TicketCheckIP         => 1,
+    TicketCheckBrowser    => 0
 );
 
 # configured items get dumped in here
@@ -70,12 +68,12 @@ sub configure {
 sub authen_cred {
     my ($class, $r, @cred) = @_;
 
-    my $this = $class->new($r);
+    my $self = $class->new($r);
 
     my ($user, $pass) = @cred;
-    my ($result, $msg) = $this->check_credentials($user, $pass);
+    my ($result, $msg) = $self->check_credentials($user, $pass);
     if ($result) {
-        return $this->make_ticket($r, $user);
+        return $self->make_ticket($r, $user);
     }
     else {
         return undef;
@@ -87,9 +85,9 @@ sub authen_cred {
 sub authen_ses_key {
     my ($class, $r, $session_key) = @_;
 
-    my $this = $class->new($r);
-    if ($this->verify_ticket($session_key)) {
-        my %ticket = $this->_unpack_ticket($session_key);
+    my $self = $class->new($r);
+    if ($self->verify_ticket($session_key)) {
+        my %ticket = $self->_unpack_ticket($session_key);
         return $ticket{user};
     } else {
         return undef;
@@ -111,9 +109,11 @@ sub _get_config_item {
 
     my $auth_name = $r->auth_name;
 
-    my $value = $r->dir_config("${auth_name}$item") ||
-           $CONFIG{$auth_name}->{$item} ||
-           $DEFAULTS{$item};
+    my $value = Apache::AuthTicket::Util::str_config_value(
+        $r->dir_config("${auth_name}$item"),
+        $CONFIG{$auth_name}->{$item},
+        $DEFAULTS{$item});
+
     warn "returning [$value] for $item" if DEBUGGING;
     return $value;
 }
@@ -159,10 +159,6 @@ sub make_login_screen {
         q{<H1>Please Log In</H1>}
     );
 
-    #if (defined $msg and $msg) {
-    #    $r->print(qq{<h2><font color="#ff0000">Error: $msg</font></h2>});
-    #}
-
     $r->print(
         qq{<form method="post" action="$action">},
         qq{<input type="hidden" name="destination" value="$destination">},
@@ -191,16 +187,13 @@ sub make_login_screen {
 sub logout ($$) {
     my ($class, $r) = @_;
 
-    if (lc $r->dir_config('Filter') eq 'on') {
-        $r->filter_register;
-    }
+    my $self = $class->new($r);
 
-    my $this = $class->new($r);
+    $self->delete_ticket($r);
+    $self->SUPER::logout($r);
 
-    $this->delete_ticket($r);
-    $this->SUPER::logout($r);
+    $r->err_headers_out->add('Location' => $self->{TicketLogoutURI});
 
-    $r->err_headers_out->add('Location' => $this->{TicketLogoutURI});
     return REDIRECT;
 }
 
@@ -209,27 +202,25 @@ sub new {
     my ($class, $r) = @_;
     $class = ref $class || $class;
 
-    my $this = bless {
+    my $self = bless {
         _REQUEST => $r
     }, $class;
 
-    $this->init($r);
+    $self->init($r);
 
-    #warn "After init I look like this\n";
-    #warn Dumper($this), "\n";
-
-    return $this;
+    return $self;
 }
 
 sub init {
-    my ($this, $r) = @_;
-    $this->{_DBH} = $this->dbi_connect;
+    my ($self, $r) = @_;
+
+    $self->{_DBH} = $self->dbi_connect;
 
     my $auth_name = $r->auth_name;
 
     # initialize configuration
     map {
-        $this->{$_} = $this->_get_config_item($r, $_);
+        $self->{$_} = $self->_get_config_item($r, $_);
     } keys %DEFAULTS;
 }
 
@@ -237,14 +228,13 @@ sub request { shift->{_REQUEST} }
 sub dbh     { shift->{_DBH} }
 
 sub dbi_connect {
-    my ($this) = @_;
-    $this->_log_entry if DEBUGGING;
+    my $self = shift;
 
-    my $r         = $this->request;
+    my $r         = $self->request;
     my $auth_name = $r->auth_name;
 
     my ($db, $user, $pass) = map {
-        $this->_get_config_item($r, $_)
+        $self->_get_config_item($r, $_)
     } qw/TicketDB TicketDBUser TicketDBPassword/;
 
     my $dbh = DBI->connect_cached($db, $user, $pass)
@@ -257,17 +247,16 @@ sub dbi_connect {
 #
 # return true if a username exists.
 sub check_user {
-    my ($this, $user) = @_;
-    $this->_log_entry if DEBUGGING;
+    my ($self, $user) = @_;
 
-    my $dbh = $this->dbh;
+    my $dbh = $self->dbh;
 
     my $rows = 0;
 
-    my ($table, $user_field) = split(/:/, $this->{TicketUserTable});
+    my ($table, $user_field) = split(/:/, $self->{TicketUserTable});
 
     my ($stmt, @bind) =
-        $this->sql->select($table, 'COUNT(*)', {$user_field => $user});
+        $self->sql->select($table, 'COUNT(*)', {$user_field => $user});
 
     eval {
         ($rows) = $dbh->selectrow_array($stmt, undef, @bind);
@@ -284,16 +273,15 @@ sub check_user {
 #
 # return the password associated with a user
 sub get_password {
-    my ($this, $user) = @_;
-    $this->_log_entry if DEBUGGING;
+    my ($self, $user) = @_;
 
-    my $dbh = $this->dbh;
+    my $dbh = $self->dbh;
 
     my ($table, $user_field, $passwd_field) = 
-        split(/:/, $this->{TicketUserTable});
+        split(/:/, $self->{TicketUserTable});
 
     my ($stmt, @bind) =
-        $this->sql->select($table, [$passwd_field], {$user_field => $user});
+        $self->sql->select($table, [$passwd_field], {$user_field => $user});
 
     my $passwd = undef;
     eval {
@@ -308,38 +296,25 @@ sub get_password {
 }
 
 sub check_credentials {
-    my ($this, $user, $password) = @_;
-    $this->_log_entry if DEBUGGING;
+    my ($self, $user, $password) = @_;
 
     my ($table, $user_field, $pass_field) = 
-        split(/:/, $this->{TicketUserTable});
+        split(/:/, $self->{TicketUserTable});
 
-    my $dbh = $this->dbh;
+    my $dbh = $self->dbh;
 
     return (undef, "Can't open database: $DBI::errstr") unless $dbh;
 
-    return (undef, "invalid account") unless $this->check_user($user);
+    return (undef, "invalid account") unless $self->check_user($user);
 
     # we might add an option for crypt or MD5 style password someday
-    my $saved_passwd = $this->get_password($user);
+    my $saved_passwd = $self->get_password($user);
 
-    my $result = 0;
+    my $style = $self->{TicketPasswordStyle};
 
-    my $style = $this->{TicketPasswordStyle};
-    if ($style eq 'cleartext') {
-        $result = $this->_compare_password_cleartext($password, $saved_passwd);
+    unless (compare_password($style, $password, $saved_passwd)) {
+        return (undef, 'password mismatch')
     }
-    elsif ($style eq 'crypt') {
-        $result = $this->_compare_password_crypt($password, $saved_passwd);
-    }
-    elsif ($style eq 'md5') {
-        $result = $this->_compare_password_md5($password, $saved_passwd);
-    }
-    else {
-        die "unknown TicketPasswordStyle: $style\n";
-    }
-
-    return (undef, "password mismatch") unless $result;
 
     # its valid.
     return (1, '');
@@ -350,22 +325,21 @@ sub check_credentials {
 # ($secret, $version) = $obj->fetch_secret($ver);
 #
 sub fetch_secret {
-    my ($this, $version) = @_;
-    $this->_log_entry if DEBUGGING;
+    my ($self, $version) = @_;
 
-    my $dbh = $this->dbh;
+    my $dbh = $self->dbh;
 
     my ($secret_table, $secret_field, $secret_version_field) =
-        split(/:/, $this->{TicketSecretTable});
+        split(/:/, $self->{TicketSecretTable});
 
     unless (defined $version) {
-        $version = $this->_get_max_secret_version;
+        $version = $self->_get_max_secret_version;
     }
 
     # generate SQL
     my @fields = ($secret_field, $secret_version_field);
     my %where = ( $secret_version_field => $version );
-    my ($stmt, @bind) = $this->sql->select($secret_table, \@fields, \%where);
+    my ($stmt, @bind) = $self->sql->select($secret_table, \@fields, \%where);
 
     my ($secret, $ret_version) = (undef, undef);
     eval {
@@ -384,18 +358,24 @@ sub fetch_secret {
 # also, put the cookie in the outgoing headers so it wil be set on the client
 #
 sub make_ticket {
-    my ($this, $r, $user_name) = @_;
-    $this->_log_entry if DEBUGGING;
+    my ($self, $r, $user_name) = @_;
 
-    my $now      = time();
-    my $expires  = $now + $this->{TicketExpires} * 60;
-    my $ip       = $r->connection->remote_ip;
-    my ($secret, $sec_version) = $this->fetch_secret();
+    my $now     = time;
+    my $expires = $now + $self->{TicketExpires} * 60;
+    my ($secret, $sec_version) = $self->fetch_secret();
 
-    my $hash = md5_hex($secret .
-                   md5_hex(join ':', $secret, $ip, $sec_version, 
-                                      $now, $expires, $user_name)
-               );
+    my @fields = ($secret, $sec_version, $now, $expires, $user_name);
+
+    # only add ip if TicketCheckIP is on.
+    if ($self->_get_config_item($r, 'TicketCheckIP')) {
+        push @fields, $r->connection->remote_ip;
+    }
+
+    if ($self->_get_config_item($r, 'TicketCheckBrowser')) {
+        push @fields, Apache::AuthTicket::Util::user_agent($r);
+    }
+
+    my $hash = Apache::AuthTicket::Util::hash_for(@fields);
 
     my %key = (
         'version' => $sec_version,
@@ -406,7 +386,7 @@ sub make_ticket {
     );
 
     eval {
-        $this->save_hash($key{'hash'});
+        $self->save_hash($key{'hash'});
     };
     if ($@) {
         warn "save_hash() failed, treating this request as invalid login.\n";
@@ -414,20 +394,19 @@ sub make_ticket {
         return;
     }
 
-    return $this->_pack_ticket(%key);
+    return $self->_pack_ticket(%key);
 }
 
 # invalidate the ticket by expiring the cookie, and delete the hash locally
 sub delete_ticket {
-    my ($this, $r) = @_;
-    $this->_log_entry if DEBUGGING;
+    my ($self, $r) = @_;
 
-    my $key = $this->key($r);
+    my $key = $self->key($r);
     warn "delete_ticket: key $key" if DEBUGGING;
 
-    my %ticket = $this->_unpack_ticket($key);
+    my %ticket = $self->_unpack_ticket($key);
 
-    $this->delete_hash($ticket{'hash'});
+    $self->delete_hash($ticket{'hash'});
 }
 
 #
@@ -436,10 +415,9 @@ sub delete_ticket {
 # return true if the ticket contains the required fields.
 #
 sub check_ticket_format {
-    my ($this, %key) = @_;
-    $this->_log_entry if DEBUGGING;
+    my ($self, %key) = @_;
 
-    $this->request->log_error("key is ".join(' ', %key)) if DEBUGGING;
+    $self->request->log_error("key is ".join(' ', %key)) if DEBUGGING;
     for my $param (qw(version time user expires hash)) {
         return 0 unless defined $key{$param};
     }
@@ -463,20 +441,19 @@ sub _pack_ticket {
 # Verify the ticket and return true or false.
 #
 sub verify_ticket {
-    my ($this, $key) = @_;
-    $this->_log_entry if DEBUGGING;
+    my ($self, $key) = @_;
 
-    my $r = $this->request;
+    my $r = $self->request;
 
     warn "ticket is $key\n" if DEBUGGING;
     my ($secret, $sec_version);
-    my %ticket = $this->_unpack_ticket($key);
+    my %ticket = $self->_unpack_ticket($key);
 
-    unless ($this->check_ticket_format(%ticket)) {
+    unless ($self->check_ticket_format(%ticket)) {
         $r->subprocess_env(AuthTicketReason => 'malformed_ticket');
         return 0;
     }
-    unless ($this->is_hash_valid($ticket{'hash'})) {
+    unless ($self->is_hash_valid($ticket{'hash'})) {
         $r->subprocess_env(AuthTicketReason => 'invalid_hash');
         return 0;
     }
@@ -484,25 +461,35 @@ sub verify_ticket {
         $r->subprocess_env(AuthTicketReason => 'expired_ticket');
         return 0;
     }
-    unless (($secret, $sec_version) = $this->fetch_secret($ticket{'version'})) {
-        # cat get server secret
+    unless (($secret, $sec_version) = $self->fetch_secret($ticket{'version'})) {
+        # can't get server secret
         $r->subprocess_env(AuthTicketReason => 'missing_secret');
         return 0;
     }
-    if ($this->_ticket_idle_timeout($ticket{'hash'})) {
+    if ($self->_ticket_idle_timeout($ticket{'hash'})) {
         # user has exceeded idle-timeout
         $r->subprocess_env(AuthTicketReason => 'idle_timeout');
-        $this->delete_hash($ticket{'hash'});
+        $self->delete_hash($ticket{'hash'});
         return 0;
     }
 
     # create a new hash and verify that it matches the supplied hash
     # (prevents tampering with the cookie)
-    my $ip = $r->connection->remote_ip;
-    my $newhash = md5_hex($secret .
-                      md5_hex(join ':', $secret, $ip,
-                          @ticket{qw(version time expires user)})
-                  );
+
+    my @fields = ($secret, @ticket{qw(version time expires user)});
+
+    if ($self->_get_config_item($r, 'TicketCheckIP')) {
+        my $ip = $r->connection->remote_ip;
+        push @fields, $ip;
+    }
+
+    if ($self->_get_config_item($r, 'TicketCheckBrowser')) {
+        push @fields, Apache::AuthTicket::Util::user_agent($r);
+    }
+
+    warn "FIELDS: [@fields]\n" if DEBUGGING;
+
+    my $newhash = Apache::AuthTicket::Util::hash_for(@fields);
 
     unless ($newhash eq $ticket{'hash'}) {
         # ticket hash does not match (ticket tampered with?)
@@ -511,7 +498,7 @@ sub verify_ticket {
     }
 
     # otherwise, everything is ok
-    $this->_update_ticket_timestamp($ticket{'hash'});
+    $self->_update_ticket_timestamp($ticket{'hash'});
     $r->user($ticket{'user'});
     return 1;
 }
@@ -519,29 +506,25 @@ sub verify_ticket {
 ########## SERVER SIDE HASH MANAGEMENT METHODS
 
 sub _update_ticket_timestamp {
-    my ($this, $hash) = @_;
+    my ($self, $hash) = @_;
 
-    my $time = $this->request->request_time;
-    my $dbh = $this->dbh;
+    my $time = $self->request->request_time;
+    my $dbh = $self->dbh;
 
-    my ($table, $tick_field, $ts_field) = split(':', $this->{TicketTable});
+    my ($table, $tick_field, $ts_field) = split(':', $self->{TicketTable});
 
-    my $query = qq{
-        UPDATE $table
-        SET    $ts_field = ?
-        WHERE  $tick_field = ?
-    };
+    my ($query, @bind) = $self->sql->update($table,
+        {$ts_field   => $time},
+        {$tick_field => $hash});
 
     eval {
-        my $sth = $dbh->prepare($query);
-        $sth->execute($time, $hash);
+        my $sth = $dbh->do($query, undef, @bind);
         $dbh->commit unless $dbh->{AutoCommit};
     };
     if ($@) {
         $dbh->rollback;
         die $@;
     }
-
 }
 
 # boolean _ticket_idle_timeout(String hash)
@@ -549,13 +532,13 @@ sub _update_ticket_timestamp {
 # return true if the ticket table timestamp is older than the IdleTimeout
 # value.
 sub _ticket_idle_timeout {
-    my ($this, $hash) = @_;
+    my ($self, $hash) = @_;
 
-    my $idle = $this->{TicketIdleTimeout} * 60;
+    my $idle = $self->{TicketIdleTimeout} * 60;
     return 0 unless $idle;       # if not timeout set, its still valid.
 
-    my $db_time = $this->{DBTicketTimeStamp};
-    my $time = $this->request->request_time;
+    my $db_time = $self->{DBTicketTimeStamp};
+    my $time = $self->request->request_time;
     if (DEBUGGING) {
         warn "Last activity: ", ($time - $db_time), " secs ago\n";
         warn "Fail if thats > ", ($idle), "\n";
@@ -564,7 +547,8 @@ sub _ticket_idle_timeout {
     if ( ($time - $db_time)  > $idle ) {
         # its timed out
         return 1;
-    } else {
+    }
+    else {
         return 0;
     }
 }
@@ -573,20 +557,18 @@ sub _ticket_idle_timeout {
 # save the ticket hash in the db
 #
 sub save_hash {
-    my ($this, $hash) = @_;
-    $this->_log_entry if DEBUGGING;
+    my ($self, $hash) = @_;
 
-    my ($table, $tick_field, $ts_field) = split(/:/, $this->{TicketTable});
-    my $dbh = $this->dbh;
+    my ($table, $tick_field, $ts_field) = split(/:/, $self->{TicketTable});
 
-    my $query = qq{
-        INSERT INTO $table ($tick_field, $ts_field)
-        VALUES (?, ?)
-    };
+    my ($query, @bind) = $self->sql->insert($table, {
+        $tick_field => $hash,
+        $ts_field   => $self->request->request_time });
+
+    my $dbh = $self->dbh;
 
     eval {
-        my $sth = $dbh->prepare($query);
-        $sth->execute($hash, $this->request->request_time);
+        my $sth = $dbh->do($query, undef, @bind);
         $dbh->commit unless $dbh->{AutoCommit};
     };
     if ($@) {
@@ -599,21 +581,16 @@ sub save_hash {
 # delete the ticket hash from the db
 #
 sub delete_hash {
-    my ($this, $hash) = @_;
-    $this->_log_entry if DEBUGGING;
+    my ($self, $hash) = @_;
 
-    my ($table, $tick_field) = split(/:/, $this->{TicketTable});
-    my $dbh = $this->dbh;
+    my ($table, $tick_field) = split(/:/, $self->{TicketTable});
 
-    my $query = qq{
-        DELETE
-        FROM    $table
-        WHERE   $tick_field = ?
-    };
+    my ($query, @bind) = $self->sql->delete($table, { $tick_field => $hash });
+
+    my $dbh = $self->dbh;
 
     eval {
-        my $sth = $dbh->prepare($query);
-        $sth->execute($hash);
+        my $sth = $dbh->do($query, undef, @bind);
         $dbh->commit unless $dbh->{AutoCommit} || 0;
     };
     if ($@) {
@@ -626,24 +603,19 @@ sub delete_hash {
 # return TRUE if the hash is in the db
 #
 sub is_hash_valid {
-    my ($this, $hash) = @_;
-    $this->_log_entry if DEBUGGING;
+    my ($self, $hash) = @_;
 
-    my ($table, $tick_field, $ts_field) = split(/:/, $this->{TicketTable});
-    my $dbh = $this->dbh;
+    my ($table, $tick_field, $ts_field) = split(/:/, $self->{TicketTable});
 
-    my $query = qq{
-        SELECT  $tick_field, $ts_field
-        FROM    $table
-        WHERE   $tick_field = ?
-    };
+    my ($query, @bind) = $self->sql->select($table, [$tick_field, $ts_field], 
+        { $tick_field => $hash });
+
+    my $dbh = $self->dbh;
 
     my ($db_hash, $ts) = (undef, undef);
     eval {
-        my $sth = $dbh->prepare($query);
-        $sth->execute($hash);
-        ($db_hash, $ts) = $sth->fetchrow_array;
-        $this->{DBTicketTimeStamp} = $ts;   # cache for later use.
+        ($db_hash, $ts) = $dbh->selectrow_array($query, undef, @bind);
+        $self->{DBTicketTimeStamp} = $ts;   # cache for later use.
     };
     if ($@) {
         $dbh->rollback;
@@ -653,61 +625,19 @@ sub is_hash_valid {
     return (defined $db_hash and $db_hash eq $hash) ? 1 : 0;
 }
 
-# PRIVATE METHODS ############################################################
-
-# logs entry into methods
-sub _log_entry {
-    my ($this) = @_;
-    my ($package, $filename, $line, $subroutine) = caller(1);
-    $this->request->log_error("ENTRY $subroutine [line $line]");
-}
-
-# compare cleartext style passwords
-sub _compare_password_cleartext {
-    my ($this, $clearpass, $saved_pass) = @_;
-    $this->_log_entry if DEBUGGING;
-
-    return $clearpass eq $saved_pass;
-}
-
-# compare crypt() style passwords
-sub _compare_password_crypt {
-    my ($this, $clearpass, $saved_pass) = @_;
-    $this->_log_entry if DEBUGGING;
-
-    my $test_pass = crypt($clearpass, $saved_pass);
-    return $test_pass eq $saved_pass;
-}
-
-# compare md5 style passwords
-sub _compare_password_md5 {
-    my ($this, $clearpass, $saved_pass) = @_;
-    $this->_log_entry if DEBUGGING;
-
-    my $test_pass = md5_hex($clearpass);
-    return $test_pass eq $saved_pass;
-}
-
 sub _get_max_secret_version {
-    my ($this) = @_;
+    my ($self) = @_;
 
     my ($secret_table, $secret_field, $secret_version_field) =
-        split(/:/, $this->{TicketSecretTable});
-    
-    my $dbh = $this->dbh;
+        split(/:/, $self->{TicketSecretTable});
 
-    my $query = qq{
-        SELECT MAX($secret_version_field)
-        FROM   $secret_table
-    };
+    my ($query) = $self->sql->select($secret_table, ["MAX($secret_version_field)"]);
+
+    my $dbh = $self->dbh;
 
     my $version = undef;
     eval {
-        my $sth = $dbh->prepare($query);
-        $sth->execute;
-        $sth->bind_columns(\$version);
-        $sth->fetch;
-        $sth->finish;
+        ($version) = $dbh->selectrow_array($query);
     };
     if ($@) {
         $dbh->rollback;
@@ -723,12 +653,12 @@ __END__
 
 =head1 NAME
 
-Apache::AuthTicket - Cookie based access module.
+Apache2::AuthTicket - Cookie based access module.
 
 =head1 SYNOPSIS
 
  # in httpd.conf
- PerlModule Apache::AuthTicket
+ PerlModule Apache2::AuthTicket
  PerlSetVar FooTicketDB DBI:mysql:database=mschout;host=testbed
  PerlSetVar FooTicketDBUser test
  PerlSetVar FooTicketDBPassword secret
@@ -744,12 +674,12 @@ Apache::AuthTicket - Cookie based access module.
  PerlSetVar FooDomain .foo.com
  PerlSetVar FooSecure 1
  PerlSetVar FooLoginScript /foologinform
- 
+
  <Location /foo>
-     AuthType Apache::AuthTicket
+     AuthType Apache2::AuthTicket
      AuthName Foo
-     PerlAuthenHandler Apache::AuthTicket->authenticate
-     PerlAuthzHandler Apache::AuthTicket->authorize
+     PerlAuthenHandler Apache2::AuthTicket->authenticate
+     PerlAuthzHandler Apache2::AuthTicket->authorize
      require valid-user
  </Location>
  
@@ -780,11 +710,11 @@ This module provides ticket based access control.  The theory behind this is
 similar to the system described in the eagle book.
 
 This module works using HTTP cookies to check if a user is authorized to view a
-page.  I<Apache::AuthCookie> is used as the underlying mechanism for managing
+page.  I<Apache2::AuthCookie> is used as the underlying mechanism for managing
 cookies.
 
 This module was designed to be as extensible as possible.  Its quite likely
-that you will want to create your own subclass of I<Apache::AuthTicket> in
+that you will want to create your own subclass of I<Apache2::AuthTicket> in
 order to customize various aspects of this module (show your own versions of
 the forms, override database methods etc). 
 
@@ -811,7 +741,7 @@ is incorporated into the cryptographic signature. If the ticket were
 intercepted, then an attacker would have to steal the user's IP address in
 order to be able to use the ticket.  Plus, since the tickets can expire
 automatically, we can be sure that the ticket is not valid for a long period of
-time.  Finally, by using the I<Secure> mode of I<Apache::AuthCookie>, the
+time.  Finally, by using the I<Secure> mode of I<Apache2::AuthCookie>, the
 ticket is not passed over unencrypted connections.  In order to attack this
 system, an attacker would have to exploit both the MD5 algorightm as well as
 SSL. Chances are, by the time the user could break both of these, the ticket
@@ -828,7 +758,7 @@ There are two things you must do in order to configure this module:
 
 There are two ways that this module could be configured.  Either by using a
 function call in startup.pl, or by configuring each handler explicitly in
-httpd.conf.  If you decide to mix and match using calls to Apache::AuthTicket->configure() with directives in httpd.conf, then remember that the following precedence applies:
+httpd.conf.  If you decide to mix and match using calls to Apache2::AuthTicket->configure() with directives in httpd.conf, then remember that the following precedence applies:
 
  o If a directive is specified in httpd.conf, it will be used.
  o else if a directive is specified by configure(), then the 
@@ -843,10 +773,10 @@ these is the block specifying your access restrictions.  This block should look
 somrthing like this:
 
  <Location /foo>
-     AuthType Apache::AuthTicket
+     AuthType Apache2::AuthTicket
      AuthName Foo
-     PerlAuthenHandler Apache::AuthTicket->authenticate
-     PerlAuthzHandler Apache::AuthTicket->authorize
+     PerlAuthenHandler Apache2::AuthTicket->authenticate
+     PerlAuthzHandler Apache2::AuthTicket->authorize
      require valid-user
  </Location>
 
@@ -854,37 +784,37 @@ The remaining blocks control how to display the login form, and the login and
 logout urls.  These blocks should look similar to this:
 
  <Location /foologinform>
-     AuthType Apache::AuthTicket
+     AuthType Apache2::AuthTicket
      AuthName Foo
      SetHandler perl-script
-     Perlhandler Apache::AuthTicket->login_screen
+     PerlResponseHandler Apache2::AuthTicket->login_screen
  </Location>
  
  <Location /foologin>
-     AuthType    Apache::AuthTicket
+     AuthType    Apache2::AuthTicket
      AuthName    Foo
      SetHandler  perl-script
-     PerlHandler Apache::AuthTicket->login
+     PerlResponseHandler Apache2::AuthTicket->login
  </Location>
  
  <Location /foo/logout>
-     AuthType Apache::AuthTicket
+     AuthType Apache2::AuthTicket
      AuthName Foo
      SetHandler perl-script
-     PerlHandler Apache::AuthTicket->logout
+     PerlResponseHandler Apache2::AuthTicket->logout
  </Location>
 
 =head2 Apache Configuration - startup.pl
 
-Any I<Apache::AuthTicket> configuration items can be set in startup.pl.  You
+Any I<Apache2::AuthTicket> configuration items can be set in startup.pl.  You
 can configure an AuthName like this:
 
- Apache::AuthTicket->configure(String auth_name, *Hash config)
+ Apache2::AuthTicket->configure(String auth_name, *Hash config)
 
 Note that when configuring this way you dont prefix the configuration items
 with the AuthName value like you do when using PerlSetVar directives.
 
-Note: You must still include I<Apache::AuthCookie> configuration directives in 
+Note: You must still include I<Apache2::AuthCookie> configuration directives in 
 httpd.conf when configuring the server this way.  These items include:
 
     PerlSetVar FooPath /
@@ -893,7 +823,7 @@ httpd.conf when configuring the server this way.  These items include:
     PerlSetVar FooLoginScript /foologinform
 
 example:
- Apache::AuthTicket->configure('Foo', {
+ Apache2::AuthTicket->configure('Foo', {
      TicketDB            => 'DBI:mysql:database=test;host=foo',
      TicketDBUser        => 'mschout',
      TicketDBPassword    => 'secret',
@@ -1004,6 +934,20 @@ they are successfully logged out (this is done via a redirect).
 
 Example: /logged_out_message.html
 
+=item B<TicketCheckIP> (default: on)
+
+This controlls whether or not the client IP address is included in the ticket
+hash.  The default is 'on'.  If you turn this off, then the client ip address
+will not be checked.  It is sometimes not desirable to check the client ip if
+the clients are behind load balancers and subsequent requests might come in
+from a different IP.
+
+=item B<TicketCheckBrowser> (default: off)
+
+This controlls whether or not the C<USER_AGENT> string is included in the
+ticket hash.  This can be used in conjunction with, or instead of
+C<TicketCheckIP> to prevent tampering with the ticket.
+
 =back
 
 =head2 Database Configuration
@@ -1055,7 +999,7 @@ This table is configured by the I<TicketSecretTable> directive.
 
 =head1 METHODS
 
-This is not a complete listing of methods contained in I<Apache::AuthTicket>.
+This is not a complete listing of methods contained in I<Apache2::AuthTicket>.
 Rather, it is a listing of methods that you might want to overload if you were
 subclassing this module.  Other methods that exist in the module are probably
 not useful to you.
@@ -1072,21 +1016,21 @@ overload this method to create your own login screen.  The log in screen only
 needs to contain a hidden field called "destination" with the contents of
 I<destination> in it, a text field named I<credential_0> and a password field
 named I<credential_1>.  You are responsible for sending the http header as well
-as the content.  See I<Apache::AuthCookie> for the description of what each of
+as the content.  See I<Apache2::AuthCookie> for the description of what each of
 these fields are for.
 
 I<action> contains the action URL for the form.  You must set the action of
 your form to this value for it to function correctly.
 
-I<Apache::AuthTicket> also provides a mechanism to determine why the login for
+I<Apache2::AuthTicket> also provides a mechanism to determine why the login for
 is being displayed.  This can be used in conjunction with
-I<Apache::AuthCookie>'s "AuthCookieReason" setting to determine why the user is
-being asked to log in.  I<Apache::AuthCookie> sets
+I<Apache2::AuthCookie>'s "AuthCookieReason" setting to determine why the user is
+being asked to log in.  I<Apache2::AuthCookie> sets
 $r->prev->subprocess_env("AuthCookieReason") to either "no_cookie" or
 "bad_cookie" when this page is loaded.  If the value is "no_cookie" then the
 user is being asked to log in for the first time, or they are logging in after
 they previously logged out.  If this value is "bad_cookie" then
-I<Apache::AuthTicket> is asking them to re-login for some reason.  To determine
+I<Apache2::AuthTicket> is asking them to re-login for some reason.  To determine
 what this reason is, you must examine
 $r->prev->subprocess_env("AuthTicketReason").  I<AuthTicketReason> can take the
 following values:
@@ -1137,7 +1081,7 @@ This method connects to the TicketDB data source. You might overload this
 method if you have a common DBI connection function. For example:
 
  sub dbi_connect {
-     my ($this) = @_;
+     my ($self) = @_;
      return Foo::dbi_connect();
  }
 
@@ -1156,16 +1100,36 @@ software, please let me know.
 The idea for this module came from the Ticket Access system in the eagle book,
 along with several ideas discussed on the mod_perl mailing list.
 
-Thanks to Ken Williams for his wonderful I<Apache::AuthCookie> module, and for
-putting in the necessary changes to I<Apache::AuthCookie> to make this module
+Thanks to Ken Williams for his wonderful I<Apache2::AuthCookie> module, and for
+putting in the necessary changes to I<Apache2::AuthCookie> to make this module
 work!
 
 =head1 AUTHOR
 
 Michael Schout <mschout@gkg.net>
 
+=head1 COPYRIGHT & LICENSE
+
+Copyright 2000-2009 Michael Schout.
+
+This program is free software; you can redistribute it and/or modify it under
+the terms of either:
+
+=over 4
+
+=item *
+
+the GNU General Public License as published by the Free Software
+Foundation; either version 1, or (at your option) any later version, or
+
+=item *
+
+the Artistic License version 2.0.
+
+=back
+
 =head1 SEE ALSO
 
-L<perl>, L<mod_perl>, L<Apache>, L<Apache::AuthCookie>
+L<perl>, L<mod_perl>, L<Apache>, L<Apache2::AuthCookie>
 
 =cut
